@@ -16,6 +16,12 @@ class HttpError(Exception):
         super(HttpError, self).__init__(self.message)
 
 
+class ChecksumError(Exception):
+    """Raised when the md5 hash of the content does not match the header."""
+
+    pass
+
+
 RETRIABLE_EXCEPTIONS = (
     requests.exceptions.ChunkedEncodingError,
     requests.exceptions.ConnectionError,
@@ -25,6 +31,7 @@ RETRIABLE_EXCEPTIONS = (
     requests.exceptions.SSLError,
     requests.exceptions.ContentDecodingError,
     google.auth.exceptions.RefreshError,
+    ChecksumError,
 )
 
 
@@ -41,3 +48,29 @@ def is_retriable(exception):
         return exception.code in errs
 
     return isinstance(exception, RETRIABLE_EXCEPTIONS)
+
+
+class FileSender:
+    def __init__(self, consistency="none"):
+        self.consistency = consistency
+        if consistency == "size":
+            self.sent = 0
+        elif consistency == "md5":
+            from hashlib import md5
+
+            self.md5 = md5()
+
+    async def send(self, pre, f, post):
+        yield pre
+        chunk = f.read(64 * 1024)
+        while chunk:
+            yield chunk
+            if self.consistency == "size":
+                self.sent += len(chunk)
+            elif self.consistency == "md5":
+                self.md5.update(chunk)
+            chunk = f.read(64 * 1024)
+        yield post
+
+    def __len__(self):
+        return self.sent
